package jugs.javafx.a_basics;

import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TreeMap;

import javafx.application.Application;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.CheckBox;
import javafx.scene.control.ListCell;
import javafx.scene.control.ListView;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

/**
 * Beispielprogramm für JavaFX-Schulung 
 * 
 * @author Michael Inden
 * 
 * Copyright 2015 by Michael Inden 
 */
public class Exercise3c_ColoredListAllColorsExample extends Application
{
    private final Map<Color, String> colorLookUpMap = sortMapByValue(fillColorLookUpMap());

    @Override
    public void start(Stage primaryStage) throws Exception
    {
        final ObservableList<Color> entries = FXCollections.observableArrayList(colorLookUpMap.keySet());

        final ListView<Color> listView = new ListView<>(entries);
        listView.setPrefHeight(200);

        // Action Handling und CSS
        final CheckBox applyStyle = new CheckBox("Apply style");
        applyStyle.setOnAction(event -> {
            final String listCss = this.getClass().getResource("list.css").toExternalForm();
            if (applyStyle.isSelected())
            {
                listView.getScene().getStylesheets().add(listCss);
            }
            else
            {
                listView.getScene().getStylesheets().remove(listCss);
            }
        });

        listView.setCellFactory(list -> new ColorRectCell(colorLookUpMap));

        final VBox vBox = new VBox();
        vBox.setPadding(new Insets(5, 5, 5, 5));
        vBox.setSpacing(10);
        vBox.getChildren().addAll(listView, applyStyle);

        primaryStage.setScene(new Scene(vBox, 300, 250));
        primaryStage.setTitle(this.getClass().getSimpleName());
        primaryStage.show();
    }

    public static void main(String[] args)
    {
        launch(args);
    }

    // "Renderer"
    static class ColorRectCell extends ListCell<Color>
    {
        private final Map<Color, String> colorLookUpMap;
        
        public ColorRectCell(final Map<Color, String> colorLookUpMap)
        {
            this.colorLookUpMap = colorLookUpMap;
        }

        @Override
        public void updateItem(final Color item, boolean empty)
        {
            super.updateItem(item, empty);
        }
    }

    // reflective way to get all predfined colors
    static Map<Color, String> fillColorLookUpMap()
    {
        final Map<Color, String> colorLookUpMap = new HashMap<>();
        
// TODO
        
        return colorLookUpMap;
    }
    
    // -------------------------------------------------------------------------
    // Special Collection Handling  
    // -------------------------------------------------------------------------

    public static Map<Color, String> sortMapByValue(final Map<Color, String> map)
    {
        class ValueComparator implements Comparator<Color>
        {
            public int compare(final Color first, final Color second)
            {
                final int result = map.get(first).compareTo(map.get(second));
                if (result == 0) // Der Wert 0 würde ungewünschte Ergebnisse liefern  
                {
                    return -1;
                }
                return result;
            }
        }

        // Sortierung
        final ValueComparator comparator = new ValueComparator();
        final TreeMap<Color, String> valueSortedMap = new TreeMap<>(comparator);
        valueSortedMap.putAll(map);
        
        // Bereitstellung als LinkedHashMap, die die Ordnung des Einfügens aufrecht erhält
        final LinkedHashMap<Color, String> valueSortedMap2 = new LinkedHashMap<>();
        valueSortedMap2.putAll(valueSortedMap);
        return valueSortedMap2;
    }
}
